@@ -160,8 +160,42 @@ namespace xt
 
         template <class B, class R>
         using functor_batch_simd_type_t = typename functor_batch_simd_type<B, R>::type;
+
+        // State storage
+        template<class S, class is_shape_trivial>
+        struct xfunction_cache_impl
+        {
+            S shape;
+            bool is_trivial;
+            bool is_initialized;
+
+            xfunction_cache_impl() : shape(xtl::make_sequence<S>(0, std::size_t(0))), is_initialized(false) {}
+        };
+
+        template<std::size_t... N, class is_shape_trivial>
+        struct xfunction_cache_impl<fixed_shape<N...>, is_shape_trivial>
+        {
+            XTENSOR_CONSTEXPR_ENHANCED_STATIC fixed_shape<N...> shape = fixed_shape<N...>();
+            XTENSOR_CONSTEXPR_ENHANCED_STATIC bool is_trivial = is_shape_trivial::value;
+            XTENSOR_CONSTEXPR_ENHANCED_STATIC bool is_initialized = true;
+        };
+
+        #ifdef XTENSOR_HAS_CONSTEXPR_ENHANCED
+            // Out of line definitions to prevent linker errors prior to C++17
+            template <std::size_t... N, class is_shape_trivial>
+            constexpr fixed_shape<N...> xfunction_cache_impl<fixed_shape<N...>, is_shape_trivial>::shape;
+
+            template <std::size_t... N, class is_shape_trivial>
+            constexpr bool xfunction_cache_impl<fixed_shape<N...>, is_shape_trivial>::is_trivial;
+
+            template <std::size_t... N, class is_shape_trivial>
+            constexpr bool xfunction_cache_impl<fixed_shape<N...>, is_shape_trivial>::is_initialized;
+        #endif
     }
 
+    template<class promote>
+    struct xfunction_cache : detail::xfunction_cache_impl<typename promote::type, promote> {};
+
     template <class F, class R, class... CT>
     class xfunction_iterator;
 
@@ -183,6 +217,8 @@ namespace xt
      * xfunction_base *
      ******************/
 
+
+
 #define DL XTENSOR_DEFAULT_LAYOUT
 
     /**
@@ -296,22 +332,14 @@ namespace xt
         using iterable_base::crbegin;
         using iterable_base::crend;
 
-        template <layout_type L = DL>
-        auto storage_begin() const noexcept;
-        template <layout_type L = DL>
-        auto storage_end() const noexcept;
-        template <layout_type L = DL>
-        auto storage_cbegin() const noexcept;
-        template <layout_type L = DL>
-        auto storage_cend() const noexcept;
+        const_storage_iterator storage_begin() const noexcept;
+        const_storage_iterator storage_end() const noexcept;
+        const_storage_iterator storage_cbegin() const noexcept;
+        const_storage_iterator storage_cend() const noexcept;
 
-        template <layout_type L = DL>
         const_reverse_storage_iterator storage_rbegin() const noexcept;
-        template <layout_type L = DL>
         const_reverse_storage_iterator storage_rend() const noexcept;
-        template <layout_type L = DL>
         const_reverse_storage_iterator storage_crbegin() const noexcept;
-        template <layout_type L = DL>
         const_reverse_storage_iterator storage_crend() const noexcept;
 
         template <class S>
@@ -349,7 +377,7 @@ namespace xt
 
         template <std::size_t... I, class... Args>
         const_reference access_impl(std::index_sequence<I...>, Args... args) const;
-        
+
         template <std::size_t... I, class... Args>
         const_reference unchecked_impl(std::index_sequence<I...>, Args... args) const;
 
@@ -374,9 +402,7 @@ namespace xt
 
         tuple_type m_e;
         functor_type m_f;
-        mutable inner_shape_type m_shape;
-        mutable bool m_shape_trivial;
-        mutable bool m_shape_computed;
+        mutable xfunction_cache<detail::promote_index<typename std::decay_t<CT>::shape_type...>> m_cache;
 
         friend class xfunction_iterator<F, R, CT...>;
         friend class xfunction_stepper<F, R, CT...>;
@@ -591,10 +617,7 @@ namespace xt
     template <class F, class R, class... CT>
     template <class Func, class... CTA, class U>
     inline xfunction_base<F, R, CT...>::xfunction_base(Func&& f, CTA&&... e) noexcept
-        : m_e(std::forward<CTA>(e)...), m_f(std::forward<Func>(f)), m_shape(xtl::make_sequence<inner_shape_type>(0, size_type(0))),
-          m_shape_computed(false)
-    {
-    }
+        : m_e(std::forward<CTA>(e)...), m_f(std::forward<Func>(f)) {}
     //@}
 
     /**
@@ -616,16 +639,18 @@ namespace xt
     template <class F, class R, class... CT>
     inline auto xfunction_base<F, R, CT...>::dimension() const noexcept -> size_type
     {
-        size_type dimension = m_shape_computed ? m_shape.size() : compute_dimension();
+        size_type dimension = m_cache.is_initialized ? m_cache.shape.size() : compute_dimension();
         return dimension;
     }
 
     template <class F, class R, class... CT>
     inline void xfunction_base<F, R, CT...>::compute_cached_shape() const
     {
-        m_shape = xtl::make_sequence<xindex_type_t<inner_shape_type>>(compute_dimension(), size_type(0));
-        m_shape_trivial = broadcast_shape(m_shape, false);
-        m_shape_computed = true;
+        static_assert(!detail::is_fixed<shape_type>::value, "Calling compute_cached_shape on fixed!");
+
+        m_cache.shape = xtl::make_sequence<xindex_type_t<inner_shape_type>>(compute_dimension(), size_type(0));
+        m_cache.is_trivial = broadcast_shape(m_cache.shape, false);
+        m_cache.is_initialized = true;
     }
 
     /**
@@ -636,13 +661,13 @@ namespace xt
     {
         xtl::mpl::static_if<!detail::is_fixed<inner_shape_type>::value>([&](auto self)
         {
-            if(!this->m_shape_computed)
+            if(!m_cache.is_initialized)
             {
                 self(this)->compute_cached_shape();
             }
         },
         [](auto /*self*/){});
-        return m_shape;
+        return m_cache.shape;
     }
 
     /**
@@ -768,10 +793,10 @@ namespace xt
     template <class S>
     inline bool xfunction_base<F, R, CT...>::broadcast_shape(S& shape, bool reuse_cache) const
     {
-        if (reuse_cache && m_shape_computed)
+        if(m_cache.is_initialized && reuse_cache)
         {
-            std::copy(m_shape.cbegin(), m_shape.cend(), shape.begin());
-            return m_shape_trivial;
+            std::copy(m_cache.shape.cbegin(), m_cache.shape.cend(), shape.begin());
+            return m_cache.is_trivial;
         }
         else
         {
@@ -796,65 +821,53 @@ namespace xt
     //@}
 
     template <class F, class R, class... CT>
-    template <layout_type L>
-    inline auto xfunction_base<F, R, CT...>::storage_begin() const noexcept
-        //  -> const_storage_iterator
+    inline auto xfunction_base<F, R, CT...>::storage_begin() const noexcept -> const_storage_iterator
     {
-        return storage_cbegin<L>();
+        return storage_cbegin();
     }
 
     template <class F, class R, class... CT>
-    template <layout_type L>
-    inline auto xfunction_base<F, R, CT...>::storage_end() const noexcept
-        //  -> const_storage_iterator
+    inline auto xfunction_base<F, R, CT...>::storage_end() const noexcept -> const_storage_iterator
     {
-        return storage_cend<L>();
+        return storage_cend();
     }
 
     template <class F, class R, class... CT>
-    template <layout_type L>
-    inline auto xfunction_base<F, R, CT...>::storage_cbegin() const noexcept
-        //  -> const_storage_iterator
+    inline auto xfunction_base<F, R, CT...>::storage_cbegin() const noexcept -> const_storage_iterator
     {
         auto f = [](const auto& e) noexcept { return detail::trivial_begin(e); };
         return build_iterator(f, std::make_index_sequence<sizeof...(CT)>());
     }
 
     template <class F, class R, class... CT>
-    template <layout_type L>
-    inline auto xfunction_base<F, R, CT...>::storage_cend() const noexcept
-        //  -> const_storage_iterator
+    inline auto xfunction_base<F, R, CT...>::storage_cend() const noexcept -> const_storage_iterator
     {
         auto f = [](const auto& e) noexcept { return detail::trivial_end(e); };
         return build_iterator(f, std::make_index_sequence<sizeof...(CT)>());
     }
 
     template <class F, class R, class... CT>
-    template <layout_type L>
     inline auto xfunction_base<F, R, CT...>::storage_rbegin() const noexcept -> const_reverse_storage_iterator
     {
-        return storage_crbegin<L>();
+        return storage_crbegin();
     }
 
     template <class F, class R, class... CT>
-    template <layout_type L>
     inline auto xfunction_base<F, R, CT...>::storage_rend() const noexcept -> const_reverse_storage_iterator
     {
-        return storage_crend<L>();
+        return storage_crend();
     }
 
     template <class F, class R, class... CT>
-    template <layout_type L>
     inline auto xfunction_base<F, R, CT...>::storage_crbegin() const noexcept -> const_reverse_storage_iterator
     {
-        return const_reverse_storage_iterator(storage_cend<L>());
+        return const_reverse_storage_iterator(storage_cend());
     }
 
     template <class F, class R, class... CT>
-    template <layout_type L>
     inline auto xfunction_base<F, R, CT...>::storage_crend() const noexcept -> const_reverse_storage_iterator
     {
-        return const_reverse_storage_iterator(storage_cbegin<L>());
+        return const_reverse_storage_iterator(storage_cbegin());
     }
 
     template <class F, class R, class... CT>
@@ -914,7 +927,7 @@ namespace xt
         XTENSOR_CHECK_DIMENSION(shape(), args...);
         return m_f(std::get<I>(m_e)(args...)...);
     }
-    
+
     template <class F, class R, class... CT>
     template <std::size_t... I, class... Args>
     inline auto xfunction_base<F, R, CT...>::unchecked_impl(std::index_sequence<I...>, Args... args) const -> const_reference
@@ -1047,13 +1060,19 @@ namespace xt
     template <class F, class R, class... CT>
     inline bool xfunction_iterator<F, R, CT...>::equal(const self_type& rhs) const
     {
-        return p_f == rhs.p_f && m_it == rhs.m_it;
+        // Optimization: no need to compare each subiterator since they all
+        // are incremented decremented together.
+        constexpr std::size_t index = xtl::mpl::find_if<is_xdummy_iterator, data_type>::value;
+        return std::get<index>(m_it) == std::get<index>(rhs.m_it);
     }
 
     template <class F, class R, class... CT>
     inline bool xfunction_iterator<F, R, CT...>::less_than(const self_type& rhs) const
     {
-        return p_f == rhs.p_f && m_it < rhs.m_it;
+        // Optimization: no need to compare each subiterator since they all
+        // are incremented decremented together.
+        constexpr std::size_t index = xtl::mpl::find_if<is_xdummy_iterator, data_type>::value;
+        return std::get<index>(m_it) < std::get<index>(rhs.m_it);
     }
 
     template <class F, class R, class... CT>
@@ -1192,7 +1211,7 @@ namespace xt
     }
 
     template <class F, class R, class... CT>
-    inline auto xfunction_stepper<F, R, CT...>::step_leading() 
+    inline auto xfunction_stepper<F, R, CT...>::step_leading()
         -> value_type
     {
         return step_leading_impl(std::make_index_sequence<sizeof...(CT)>());
