@@ -25,6 +25,7 @@
 #include <xtl/xsequence.hpp>
 
 #include "xbuilder.hpp"
+#include "xeval.hpp"
 #include "xexpression.hpp"
 #include "xgenerator.hpp"
 #include "xiterable.hpp"
@@ -58,6 +59,9 @@ namespace xt
     template <class ST, class X>
     struct xreducer_shape_type;
 
+    template <class S1, class S2>
+    struct fixed_xreducer_shape_type;
+
     template <class E, class X, class result_type>
     struct xreducer_result_container
     {
@@ -76,6 +80,12 @@ namespace xt
         using type = xtensor<result_type, sizeof...(N) - NX, L>;
     };
 
+    template <class T, std::size_t... I, layout_type L, std::size_t... X, class result_type>
+    struct xreducer_result_container<xtensor_fixed<T, xshape<I...>, L>, xshape<X...>, result_type>
+    {
+        using type = xtensor_fixed<result_type, typename fixed_xreducer_shape_type<fixed_shape<I...>, fixed_shape<X...>>::type, L>;
+    };
+
     template <class F, class E, class X>
     auto reduce_immediate(F&& f, E&& e, X&& axes)
     {
@@ -138,13 +148,13 @@ namespace xt
         std::size_t leading_ax = axes[(e.layout() == layout_type::row_major) ? axes.size() - 1 : 0];
         auto strides_finder = e.strides().begin() + static_cast<std::ptrdiff_t>(leading_ax);
         // The computed strides contain "0" where the shape is 1 -- therefore find the next none-zero number
-        std::size_t inner_stride = *strides_finder;
+        std::size_t inner_stride = static_cast<std::size_t>(*strides_finder);
         while (inner_stride == 0)
         {
             (e.layout() == layout_type::row_major) ? --strides_finder : ++strides_finder;
-            inner_stride = *strides_finder;
+            inner_stride = static_cast<std::size_t>(*strides_finder);
         }
-        std::size_t inner_loop_size = inner_stride;
+        std::size_t inner_loop_size = static_cast<std::size_t>(inner_stride);
         std::size_t outer_loop_size = e.shape()[leading_ax];
 
         // The following code merges reduction axes "at the end" (or the beginning for col_major)
@@ -169,7 +179,7 @@ namespace xt
             if (std::find(axes.begin(), axes.end(), i) == axes.end())
             {
                 // i not in axes!
-                iter_strides[i] = result.strides()[idx];
+                iter_strides[i] = static_cast<std::size_t>(result.strides()[idx]);
                 ++idx;
             }
         }
@@ -273,7 +283,7 @@ namespace xt
             while (idx_res.first != true)
             {
                 std::transform(out, out + inner_loop_size, begin, out,
-                               [merge, &init_fct, &merge_fct, &reduce_fct](auto&& v1, auto&& v2) {
+                               [merge, &init_fct, &reduce_fct](auto&& v1, auto&& v2) {
                                     return merge ?
                                         reduce_fct(v1, v2) :
                                         // cast because return type of identity function is not upcasted
@@ -489,14 +499,17 @@ namespace xt
         template <class F, class E, class X>
         inline auto reduce_impl(F&& f, E&& e, X&& axes, evaluation_strategy::lazy)
         {
-            using reducer_type = xreducer<F, const_xclosure_t<E>, xtl::const_closure_type_t<X>>;
-            return reducer_type(std::forward<F>(f), std::forward<E>(e), std::forward<X>(axes));
+            decltype(auto) normalized_axes = normalize_axis(e, std::forward<X>(axes));
+            using reducer_type = xreducer<F, const_xclosure_t<E>, xtl::const_closure_type_t<decltype(normalized_axes)>>;
+            return reducer_type(std::forward<F>(f), std::forward<E>(e), std::forward<decltype(normalized_axes)>(normalized_axes));
         }
 
+
         template <class F, class E, class X>
         inline auto reduce_impl(F&& f, E&& e, X&& axes, evaluation_strategy::immediate)
         {
-            return reduce_immediate(std::forward<F>(f), std::forward<E>(e), std::forward<X>(axes));
+            decltype(auto) normalized_axes = normalize_axis(e, std::forward<X>(axes));
+            return reduce_immediate(std::forward<F>(f), eval(std::forward<E>(e)), std::forward<decltype(normalized_axes)>(normalized_axes));
         }
     }
 
@@ -522,7 +535,7 @@ namespace xt
     template <class F, class E, class EVS, class>
     inline auto reduce(F&& f, E&& e, EVS evaluation_strategy)
     {
-        typename std::decay_t<E>::shape_type ar;
+        xindex_type_t<typename std::decay_t<E>::shape_type> ar;
         resize_container(ar, e.dimension());
         std::iota(ar.begin(), ar.end(), 0);
         return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), std::move(ar), evaluation_strategy);
@@ -532,16 +545,18 @@ namespace xt
     template <class F, class E, class I, class EVS>
     inline auto reduce(F&& f, E&& e, std::initializer_list<I> axes, EVS evaluation_strategy)
     {
-        using axes_type = std::vector<typename std::decay_t<E>::size_type>;
+        using axes_type = std::vector<std::size_t>;
+        auto ax = xt::forward_normalize<axes_type>(e, axes);
         using reducer_type = xreducer<F, const_xclosure_t<E>, axes_type>;
-        return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), xtl::forward_sequence<axes_type>(axes), evaluation_strategy);
+        return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), std::move(ax), evaluation_strategy);
     }
 #else
     template <class F, class E, class I, std::size_t N, class EVS>
     inline auto reduce(F&& f, E&& e, const I (&axes)[N], EVS evaluation_strategy)
     {
-        using axes_type = std::array<typename std::decay_t<E>::size_type, N>;
-        return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), xtl::forward_sequence<axes_type>(axes), evaluation_strategy);
+        using axes_type = std::array<std::size_t, N>;
+        auto ax = xt::forward_normalize<axes_type>(e, axes);
+        return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), std::move(ax), evaluation_strategy);
     }
 #endif
 
@@ -600,6 +615,49 @@ namespace xt
      * xreducer utils *
      ******************/
 
+    namespace detail
+    {
+        template <std::size_t X, std::size_t... I>
+        struct in
+        {
+            constexpr static bool value = xtl::disjunction<std::integral_constant<bool, X == I>...>::value;
+        };
+
+        template <std::size_t Z, class S1, class S2, class R>
+        struct fixed_xreducer_shape_type_impl;
+
+        template <std::size_t Z, std::size_t... I, std::size_t... J, std::size_t... R>
+        struct fixed_xreducer_shape_type_impl<Z, fixed_shape<I...>, fixed_shape<J...>, fixed_shape<R...>>
+        {
+            using type = std::conditional_t<in<Z, J...>::value,
+                                            typename fixed_xreducer_shape_type_impl<Z - 1, fixed_shape<I...>, fixed_shape<J...>,
+                                                                                    fixed_shape<R...>>::type,
+                                            typename fixed_xreducer_shape_type_impl<Z - 1, fixed_shape<I...>, fixed_shape<J...>,
+                                                                                    fixed_shape<detail::at<Z, I...>::value, R...>>::type>;
+        };
+
+        template <std::size_t... I, std::size_t... J, std::size_t... R>
+        struct fixed_xreducer_shape_type_impl<0, fixed_shape<I...>, fixed_shape<J...>, fixed_shape<R...>>
+        {
+            using type = std::conditional_t<in<0, J...>::value,
+                                            fixed_shape<R...>,
+                                            fixed_shape<detail::at<0, I...>::value, R...>>;
+        };
+    }
+
+    template <class S1, class S2>
+    struct fixed_xreducer_shape_type;
+
+    template <std::size_t... I, std::size_t... J>
+    struct fixed_xreducer_shape_type<fixed_shape<I...>, fixed_shape<J...>>
+    {
+        using type = typename detail::fixed_xreducer_shape_type_impl<sizeof...(I) - 1, 
+                                                                    fixed_shape<I...>,
+                                                                    fixed_shape<J...>,
+                                                                    fixed_shape<>>::type;
+    };
+
+
     // meta-function returning the shape type for an xreducer
     template <class ST, class X>
     struct xreducer_shape_type
@@ -613,6 +671,19 @@ namespace xt
         using type = std::array<I2, N1 - N2>;
     };
 
+    template <std::size_t... I, class I2, std::size_t N2>
+    struct xreducer_shape_type<fixed_shape<I...>, std::array<I2, N2>>
+    {
+        using type = std::array<I2, sizeof...(I) - N2>;
+    };
+
+    // Note adding "A" to prevent compilation in case nothing else matches 
+    template <std::size_t... I, std::size_t... J>
+    struct xreducer_shape_type<fixed_shape<I...>, fixed_shape<J...>>
+    {
+        using type = std::array<std::size_t, sizeof...(I) - sizeof...(J)>;
+    };
+
     namespace detail
     {
         template <class InputIt, class ExcludeIt, class OutputIt>
