@@ -97,6 +97,18 @@ namespace xt
     template <class... T>
     using void_t = typename make_void<T...>::type;
 
+    // This is used for non existant types (e.g. storage for some expressions
+    // like generators)
+    struct invalid_type
+    {
+    };
+
+    template <class... T>
+    struct make_invalid_type
+    {
+        using type = invalid_type;
+    };
+
     template <class T, class R>
     using disable_integral_t = std::enable_if_t<!std::is_integral<T>::value, R>;
 
@@ -672,17 +684,6 @@ namespace xt
     {
     };
 
-    template <class E, class = void>
-    struct has_simd_interface : std::false_type
-    {
-    };
-
-    template <class E>
-    struct has_simd_interface<E, void_t<decltype(std::declval<E>().template load_simd<aligned_mode>(typename E::size_type(0)))>>
-        : std::true_type
-    {
-    };
-
     /******************
      * enable_if_type *
      ******************/
