@@ -346,7 +346,7 @@ namespace xt
 
         using storage_type = xtl::mpl::eval_if_t<has_data_interface<xexpression_type>,
                                                  detail::expr_storage_type<xexpression_type>,
-                                                 detail::void_wrapper>;
+                                                 make_invalid_type<>>;
 
         using inner_strides_type = std::conditional_t<is_contiguous_view, 
                                                       typename detail::unwrap_offset_container<xexpression_type::static_layout, xexpression_inner_strides_type, integral_count<S...>()>::type,
@@ -362,6 +362,13 @@ namespace xt
         using stepper = typename iterable_base::stepper;
         using const_stepper = typename iterable_base::const_stepper;
 
+        using storage_iterator = std::conditional_t<has_data_interface<xexpression_type>::value,
+                                                    typename xexpression_type::storage_iterator,
+                                                    typename iterable_base::storage_iterator>;
+        using const_storage_iterator = std::conditional_t<has_data_interface<xexpression_type>::value,
+                                                    typename xexpression_type::const_storage_iterator,
+                                                    typename iterable_base::const_storage_iterator>;
+
         using container_iterator = pointer;
         using const_container_iterator = const_pointer;
 
@@ -459,6 +466,22 @@ namespace xt
         std::enable_if_t<has_data_interface<T>::value, const typename T::storage_type&>
         storage() const;
 
+        template <class T = xexpression_type>
+        std::enable_if_t<has_data_interface<T>::value && is_strided_view, storage_iterator>
+        storage_begin();
+
+        template <class T = xexpression_type>
+        std::enable_if_t<has_data_interface<T>::value && is_strided_view, storage_iterator>
+        storage_end();
+
+        template <class T = xexpression_type>
+        std::enable_if_t<has_data_interface<T>::value && is_strided_view, const_storage_iterator>
+        storage_cbegin() const;
+        
+        template <class T = xexpression_type>
+        std::enable_if_t<has_data_interface<T>::value && is_strided_view, const_storage_iterator>
+        storage_cend() const;
+
         template <class T = xexpression_type>
         std::enable_if_t<has_data_interface<T>::value && is_strided_view, const inner_strides_type&>
         strides() const;
@@ -520,16 +543,20 @@ namespace xt
         template <class simd>
         using simd_return_type = xsimd::simd_return_type<value_type, typename simd::value_type>;
 
-        template <class align, class simd = simd_value_type, class T = xexpression_type,
-                  class = std::enable_if_t<has_simd_interface<T>::value && is_strided_view, int>>
-        void store_simd(size_type i, const simd& e);
-        template <class align, class simd = simd_value_type, class T = xexpression_type,
-                  class = std::enable_if_t<has_simd_interface<T>::value && is_strided_view, int>>
-        simd_return_type<simd> load_simd(size_type i) const;
-        template <class T = xexpression_type, class = std::enable_if_t<has_simd_interface<T>::value && is_strided_view, int>>
-        inline reference data_element(size_type i);
-        template <class T = xexpression_type, class = std::enable_if_t<has_simd_interface<T>::value && is_strided_view, int>>
-        inline const_reference data_element(size_type i) const;
+        template <class T, class R>
+        using enable_simd_interface = std::enable_if_t<has_simd_interface<T>::value && is_strided_view, R>;
+
+        template <class align, class simd = simd_value_type, class T = xexpression_type>
+        enable_simd_interface<T, void> store_simd(size_type i, const simd& e);
+
+        template <class align, class simd = simd_value_type, class T = xexpression_type>
+        enable_simd_interface<T, simd_return_type<simd>> load_simd(size_type i) const;
+
+        template <class T = xexpression_type>
+        enable_simd_interface<T, reference> data_element(size_type i);
+
+        template <class T = xexpression_type>
+        enable_simd_interface<T, const_reference> data_element(size_type i) const;
 
     private:
 
@@ -817,7 +844,7 @@ namespace xt
     template <class E>
     inline auto xview<CT, S...>::operator=(const E& e) -> disable_xexpression<E, self_type>&
     {
-        std::fill(this->begin(), this->end(), e);
+        this->fill(e);
         return *this;
     }
 
@@ -899,7 +926,14 @@ namespace xt
     template <class T>
     inline void xview<CT, S...>::fill(const T& value)
     {
-        std::fill(this->storage_begin(), this->storage_end(), value);
+        if (layout() != layout_type::dynamic)
+        {
+            std::fill(this->storage_begin(), this->storage_end(), value);
+        }
+        else
+        {
+            std::fill(this->begin(), this->end(), value);
+        }
     }
 
     /**
@@ -1118,6 +1152,38 @@ namespace xt
         return m_e.storage();
     }
 
+    template <class CT, class... S>
+    template <class T>
+    auto xview<CT, S...>::storage_begin()
+        -> std::enable_if_t<has_data_interface<T>::value && is_strided_view, storage_iterator>
+    {
+        return m_e.storage_begin() + data_offset();
+    }
+
+    template <class CT, class... S>
+    template <class T>
+    auto xview<CT, S...>::storage_end()
+        -> std::enable_if_t<has_data_interface<T>::value && is_strided_view, storage_iterator>
+    {
+        return m_e.storage_begin() + data_offset() + size();
+    }
+
+    template <class CT, class... S>
+    template <class T>
+    auto xview<CT, S...>::storage_cbegin() const
+        -> std::enable_if_t<has_data_interface<T>::value && is_strided_view, const_storage_iterator>
+    {
+        return m_e.storage_cbegin() + data_offset();
+    }
+
+    template <class CT, class... S>
+    template <class T>
+    auto xview<CT, S...>::storage_cend() const
+        -> std::enable_if_t<has_data_interface<T>::value && is_strided_view, const_storage_iterator>
+    {
+        return m_e.storage_cbegin() + data_offset() + size();
+    }
+
     /**
      * Return the strides for the underlying container of the view.
      */
@@ -1309,29 +1375,29 @@ namespace xt
     }
 
     template <class CT, class... S>
-    template <class align, class simd, class, class>
-    void xview<CT, S...>::store_simd(size_type i, const simd& e)
+    template <class align, class simd, class T>
+    inline auto xview<CT, S...>::store_simd(size_type i, const simd& e) -> enable_simd_interface<T, void>
     {
         return m_e.template store_simd<xsimd::unaligned_mode>(data_offset() + i, e);
     }
 
     template <class CT, class... S>
-    template <class align, class simd, class, class>
-    auto xview<CT, S...>::load_simd(size_type i) const -> simd_return_type<simd>
+    template <class align, class simd, class T>
+    inline auto xview<CT, S...>::load_simd(size_type i) const -> enable_simd_interface<T, simd_return_type<simd>>
     {
         return m_e.template load_simd<xsimd::unaligned_mode, simd>(data_offset() + i);
     }
 
     template <class CT, class... S>
-    template <class, class>
-    inline auto xview<CT, S...>::data_element(size_type i) -> reference
+    template <class T>
+    inline auto xview<CT, S...>::data_element(size_type i) -> enable_simd_interface<T, reference>
     {
         return m_e.data_element(data_offset() + i);
     }
 
     template <class CT, class... S>
-    template <class, class>
-    inline auto xview<CT, S...>::data_element(size_type i) const -> const_reference
+    template <class T>
+    inline auto xview<CT, S...>::data_element(size_type i) const -> enable_simd_interface<T, const_reference>
     {
         return m_e.data_element(data_offset() + i);
     }
